function model=kpls_cv(varargin)
if nargin==1 && strcmp(varargin{1}, 'options')
    model.prettype={'mean' 'mean'};
    model.kerneltype='gauss';
    model.kernelpar=10.^(-2:0.5:2);
    model.cvtype='mc';
    model.cvsegments=100;
    model.cvfract=0.20;
    model.LVmax=10;
    
    return
elseif nargin==2
    X=varargin{1};
    Y=varargin{2};
    opt.prettype={'mean' 'mean'};
    opt.kerneltype='gauss';
    opt.kernelpar=10.^(-2:0.5:2);
    opt.cvtype='mc';
    opt.cvsegments=100;
    opt.cvfract=0.20;
    opt.LVmax=10;
elseif nargin==3
    X=varargin{1};
    Y=varargin{2};
    opt=varargin{3};
end

ntot=size(X,1);
%%% Begins cross-validation
cvtype=opt.cvtype;
segments=opt.cvsegments;



if strcmp(cvtype, 'loo')  %Loo cross-validation corresponds to syst123 with N segments
    cvtype='syst123';
    segments=ntot;
end
if isvector(segments)
    cvseg=segments; % A cell array
    segments=max(segments); % Now a scalar
end

npar=length(opt.kernelpar);
ErrMatr=zeros(npar,opt.LVmax);

for j=1:segments %Beginning of the crossvalidation loop
    
    fprintf('Now calculating the %d cross-validation segment \n', j)
    
    
    switch cvtype
        case 'syst123'    %venetian blind cross-validation
            t=j:segments:ntot; %Calibration set
            m=1:ntot; m(t)=[]; %Validation set
            
        case 'syst111'    %contiguous blocks
            ns=ceil(ntot./segments);  %number of samples in each group
            if j==segments
                t=(j-1)*ns+1:ntot; %Validation set
            else
                t=(j-1)*ns+1:j*ns; %Validation set
            end
            m=[1:(j-1)*ns ns*j+1:ntot]; %Calibration set
        case 'manual'
            t=find(cvseg==j); %Validation set
            m=1:ntot; m(t)=[]; %Calibration set
        case 'mc'
            f=round(opt.cvfract*ntot);   %Number of samples in each internal validation split
            m=randperm(ntot);
            t=m(1:f);         %validation set
            m=m(f+1:end);     %calibration set
            ncvs=f*segments;
            
    end
    Xtr=X(m,:); Xts=X(t,:);
    Ytr=Y(m,:); Yts=Y(t,:);
    
    for i=1:opt.LVmax
        for k=1:npar
            fprintf('LV= %d \n', i)
            fprintf('Parameter= %d  \n', opt.kernelpar(k))
    
            mi=kpls_model(Xtr,Ytr, i, opt.prettype, opt.kerneltype, opt.kernelpar(k));
            pi=kpls_pred(Xts,Yts, mi);
            ErrMatr(k,i)=ErrMatr(k,i)+sum(sum((Yts-pi.Ypred).^2));
        end
    end
    
end

if strcmp(cvtype, 'mc')
    rmsecv=sqrt(ErrMatr./ncvs);
else
    rmsecv=sqrt(ErrMatr./ntot);
end

if isvector(rmsecv)
    sopt=1;
    [~,lvopt]=min(rmsecv);
else
    % Selection of optimal kernel parameters and number of LV
    [er1,s1]=min(rmsecv);
    [~,lvopt]=min(er1);     %optimal number of LVs
    sopt=s1(lvopt);     %position corresponding to the optimal value of kernel parameter
end

kparopt=opt.kernelpar(sopt); %optimal value of kernel parameter
%Calculation of optimal model
koptmodel=kpls_model(X,Y,lvopt,opt.prettype, opt.kerneltype, kparopt);

model.OptModel=koptmodel;
model.CV.rmsecv=rmsecv;
model.CV.rmsecvOpt=rmsecv(sopt,lvopt);
model.CV.LVopt=lvopt;
model.CV.kernelparOpt=kparopt;
model.Options=opt;




