function model=kpls_model(Xtr,Ytr, ncomp, pret, kerneltype, kernelpar)

[Xp, mx, sx]=preproc(Xtr, pret{1});
[Yp, my, sy]=preproc(Ytr, pret{2});

K=kpls_makekernel(Xp,[], kerneltype, kernelpar);
Kp=kpls_center(K, []);

E=Kp;
F=Yp; 

T=zeros(size(Kp,1),ncomp);
U=T;
Q=zeros(ncomp, size(Yp,2));


for i=1:ncomp
    [~,fidx]=max(std(F), [],2);
    u_old=F(:,fidx);
    crit=1;
    iter=0;
    while crit>10^-9 && iter<2000
        t=E*u_old; 
        t=t./norm(t);
        q=F'*t;
        u=F*q; 
        u=u./norm(u);
        crit=norm(u-u_old)./norm(u_old);
        u_old=u;
        iter=iter+1;
    end
    
    T(:,i)=t; 
    U(:,i)=u; 
    Q(i,:)=q;
    
    E=(eye(size(E,1))-t*t')*E*(eye(size(E,1))-t*t');
    F=F-(t*t')*F;
end

B=U/(T'*Kp*U)*T'*Yp;
R=U/(T'*Kp*U);
Yhat=Kp*B;
Ypred=undoprep(Yhat, my,sy, pret{2});

rmsec=sqrt(sum((Ypred-Ytr).^2)/size(Ytr,1));
bias=sum(Ypred-Ytr)/size(Ytr,1);
vy=sum((Ytr-repmat(my, size(Ytr,1),1)).^2);
r2=1-(sum((Ypred-Ytr).^2)./vy);

model.Xscores=T; 
model.Yscores=U; 
model.Yloads=Q; 
model.regcoef=B; 
model.Rweights=R; 
model.Ypred=Ypred; 
model.rmsec=rmsec; 
model.bias=bias; 
model.r2=r2;
model.preptype=pret;
model.preppars={mx sx; my sy};
model.kerneltype=kerneltype; 
model.kernelpar=kernelpar; 
model.Xdata=Xtr;
model.Xres=(eye(size(Kp,1))-(T*T'))*Kp*(eye(size(Kp,1))-(T*T'));
model.Xloads=T'*Kp;















function [Xp, mx, sx]=preproc(X,prettype)

switch prettype
    case 'none'
        Xp=X;
        mx=[];
        sx=[];
    case 'mean'
        mx=mean(X);
        sx=[];
        Xp=X-repmat(mx, size(X,1),1);
    case 'auto'
        mx=mean(X);
        sx=std(X);
        Xp=(X-repmat(mx, size(X,1),1))./repmat(sx, size(X,1),1);    
    case 'paretomc'
        mx=mean(X);
        sx=sqrt(std(X));
        Xp=(X-repmat(mx, size(X,1),1))./repmat(sx, size(X,1),1);    
    case 'uv'
        mx=[];
        sx=std(X);
        Xp=X./repmat(sx, size(X,1),1);
    case 'paretosc'
        mx=[];
        sx=sqrt(std(X));
        Xp=X./repmat(sx, size(X,1),1);
    
end


function Xp=undoprep(X,mx,sx,prettype)

switch prettype
    case 'none'
        Xp=X;
    case 'mean'
        Xp=X+repmat(mx, size(X,1),1);
    case 'auto'
        Xp=(X.*repmat(sx, size(X,1),1))+repmat(mx, size(X,1),1);    
    case 'paretomc'
        Xp=(X.*repmat(sx, size(X,1),1))+repmat(mx, size(X,1),1);    
    case 'uv'
        Xp=X.*repmat(sx, size(X,1),1);
    case 'paretosc'
        Xp=X.*repmat(sx, size(X,1),1);
end